"""
setup.py
written in Python3
author: C. Lockhart <chris@lockhartlab.org>
"""

# import setuptools  # noqa
import numpy as np
from numpy.distutils.core import Extension, setup
from numpy.distutils.misc_util import Configuration
import os.path

# Read version
with open('version.yml', 'r') as f:
    data = f.read().splitlines()
version_dict = dict([element.split(': ') for element in data])

# Convert the version_data to a string
version = '.'.join([str(version_dict[key]) for key in ['major', 'minor']])
if version_dict['micro'] != 0:
    version += '.' + version_dict['micro']
print(version)

# Read in requirements.txt
with open('requirements.txt', 'r') as buffer:
    requirements = buffer.read().splitlines()

# Long description
with open('README.rst', 'r') as buffer:
    long_description = buffer.read()


# First make sure numpy is installed
# _setup(install_requires=['numpy'])

# Create configuration
def configuration(parent_package='', top_path=None):
    config = Configuration('pathogen', parent_package, top_path)
    # config.add_data_dir(('_include', 'pathetic/_include'))  # not sure why this wasn't working with manifest.in
    return config


# Then, install molecular
setup(
    # name='pathogen',
    version=version,
    author='C. Lockhart',
    author_email='chris@lockhartlab.org',
    description='Helper functions for paths',
    long_description=long_description,
    url="https://www.lockhartlab.org",
    packages=[
        'pathogen',
    ],
    install_requires=requirements,
    # include_package_data=True,
    configuration=configuration,
    zip_safe=True,
)




from glob import glob
from itertools import product
import os


class Path:
    """
    A way to hold a path (as string) but retain metadata.
    """

    __slots__ = ('_path', '_metadata')

    def __init__(self, path, **metadata):
        if isinstance(path, Path):
            metadata = path._metadata
            path = path._path

        self._path = path
        self._metadata = metadata

    def __repr__(self):
        return self._path

    def __fspath__(self):
        return self._path

    @property
    def metadata(self):
        return self._metadata


# Variable glob
def vglob(path, errors='raise', **kwargs):
    """
    Variable glob.

    Parameters
    ----------
    path : str
    errors : str
        How to handle errors? Currently only 'raise' is supported.

    Returns
    -------
    list
    """

    # Where any kwargs supplied? If not, short-circuit and glob
    if len(kwargs) == 0:
        return glob(path)

    # Variables to iterate
    keys = kwargs.keys()
    if 'path' in keys:
        raise AttributeError('`path` is protected and cannot be listed as a variable')
    if errors.lower() in 'raise':
        for key in keys:
            if key not in path:
                raise AttributeError('{' + f'{key}' + '}' + f' not in path="{path}"')

    # Values
    def _convert_to_list(value):
        # BUGFIX https://github.com/LockhartLab/molecular/issues/2#issue-838289328
        if not isinstance(value, range) and not hasattr(value, '__getitem__'):
            value = [value]
        return value

    values = map(_convert_to_list, kwargs.values())

    # Go through each set of values and
    files = []
    for value_set in product(*values):
        fmt = {key: value_set[i] for i, key in enumerate(keys)}
        fname = path.format(**fmt)
        if errors.lower() in 'raise' and not os.path.exists(fname):
            raise FileNotFoundError(fname)
        files.append(Path(fname, **fmt))

    # Return
    return files
